@@ -11,6 +11,8 @@ import os
 import os.path
 import sys
 import glob
+import pylons
+import pylons.util as util
 
 from paste.script.command import Command, BadCommand
 from paste.script.filemaker import FileOp
@@ -39,6 +41,7 @@ class ControllerCommand(Command):
         Creating yourproj/controllers/admin
         Creating yourproj/yourproj/controllers/admin/trackback.py
         Creating yourproj/yourproj/tests/functional/test_admin_trackback.py
+    
     """
     summary = __doc__
     usage = 'CONTROLLER_NAME'
@@ -47,27 +50,50 @@ class ControllerCommand(Command):
     max_args = 1
     group_name = 'pylons'
     
+    default_verbosity = 3
+    
     parser = Command.standard_parser(simulate=True)
     parser.add_option('--no-test',
                       action='store_true',
                       dest='no_test',
                       help="Don't create the test; just the controller")
-
+    
+    def validate_name(self, name):
+        if not name:
+            # This happens when the name is an existing directory
+            raise BadCommand('Please give the name of a controller.')
+        try:
+            __import__(name)
+            raise BadCommand("\n\nA module named '%s' is already present in your PYTHON_PATH.\n"
+                             "Choosing a conflicting name will likely cause import problems in\n"
+                             "your controller at some point. It's suggested that you choose an alternate\n"
+                             "name, and if you'd like that name to be accessible as '%s', add a route\n"
+                             "to your projects config/routing.py file similar to:\n"
+                             "  map.connect('%s', controller='my_%s')" % (name, name, name, name))
+        except ImportError:
+            # This is actually the result we want
+            pass
+        
+        return True
+        
     def command(self):
         try:
-            self.verbose = 3
             fo = FileOp(source_dir=os.path.join(os.path.dirname(__file__), 'templates'))
             try:
                 name, dir = fo.parse_path_name_args(self.args[0])
             except:
                 raise BadCommand('No egg_info directory was found')
+            
+            # Validate the name
+            name = name.replace('-', '_')
+            self.validate_name(name)
+            
             fullname = os.path.join(dir, name)
+            controller_name = util.class_name_from_module_name(name.split('/')[-1])
             if not fullname.startswith(os.sep): fullname = os.sep + fullname
             testname = fullname.replace(os.sep, '_')[1:]
-            fo.template_vars.update({'name': name.title().replace('-', '_'),
-                                  'fullname': fullname,
-                                  'fname': os.path.join(dir, name),
-                                  'lname': name})
+            fo.template_vars.update({'name': controller_name,
+                                  'fname': os.path.join(dir, name)})
             fo.copy_file(template='controller.py_tmpl',
                          dest=os.path.join('controllers', dir), filename=name)
             if not self.options.no_test:
@@ -142,21 +168,32 @@ class ShellCommand(Command):
         
         make_map = getattr(sys.modules[routing_package], 'make_map')
         mapper = make_map()
+        test_app = app=paste.fixture.TestApp(wsgiapp)
+        g = None
+        try:
+            g = test_app.get('/').g
+        except:
+            pass
         
         locs.update(
             dict(
                 model=sys.modules[models_package],
                 mapper=mapper,
                 wsgiapp=wsgiapp,
-                app=paste.fixture.TestApp(wsgiapp),
+                app=test_app,
                 h=sys.modules[helpers_package],
             )
         )
-                
+        if g:
+            locs['g'] = g
+            pylons.g.push_object(g)
+        
         banner = "Pylons Interactive Shell\nPython %s\n\n" % sys.version
         banner += "Additional Objects:\n"
         banner += "  %-10s -  %s\n" % ('mapper', 'Routes mapper object')
         banner += "  %-10s -  %s\n" % ('h', 'Helper object')
+        if g:
+            banner += "  %-10s -  %s\n" % ('g', 'Globals object')
         banner += "  %-10s -  %s\n" % ('model', 'Models from models package')
         banner += "  %-10s -  %s\n" % ('wsgiapp', 'This projects WSGI App instance')
         banner += "  %-10s -  %s\n" % ('app', 'paste.fixture wrapped around wsgiapp')
