@@ -42,18 +42,29 @@ class Config(object):
         be configured as the default template engine. Each item in the list is
         a dict indicating how to configure the template engine with keys:
         ``engine``, ``template_root``, ``template_options``, and ``alias``
+    ``default_charset``
+        The default character encoding specified to the browser via the
+        'charset' parameter of the HTTP response's Content-Type header.
+    ``strict_c``
+        Whether or not the ``c`` object should throw an attribute error when
+        access is attempted to an attribute that doesn't exist.
     
     """
-    def __init__(self, myghty, map, paths, environ_config=None):
+    def __init__(self, myghty, map, paths, environ_config=None, default_charset='UTF-8',
+                 strict_c=False):
         if environ_config is None:
             environ_config = {}
         self.myghty = myghty
         self.map = map
         self.paths = paths
+        self.environ_config = environ_config
+        self.default_charset = default_charset
+        self.strict_c = strict_c
+        if 'output_encoding' not in myghty:
+            myghty['output_encoding'] = default_charset
         self.global_conf = {}
         self.app_conf = {}
         self.template_engines = []
-        self.environ_config = environ_config
     
     def add_template_engine(self, engine, root, options, alias=None):
         """Add additional template engines for configuration on Pylons WSGI init
@@ -145,20 +156,6 @@ class Config(object):
         if prefix:
             self.map.prefix = app_conf['prefix']
             self.map._created_regs = False
-                
-        myghty_defaults = {}
-        
-        # Raise a complete error for the error middleware to catch
-        myghty_defaults['raise_error'] = True
-        
-        # Standard Pylons configuration directives for Myghty
-        myghty_defaults.setdefault('allow_globals', [])
-                
-        myghty_defaults['allow_globals'].extend(
-            ['c', 'h', 's', 'session', 'request', 'params', 'g', 'render', 'render_fragment']
-        )
-        myghty_defaults['component_root'] = [{os.path.basename(path): path} for \
-                                             path in self.paths['templates']]
         
         errorware = {}
         # Load the errorware configuration from the Paste configuration file
@@ -175,12 +172,28 @@ class Config(object):
                                             global_conf.get('error_email_from', 'pylons@yourapp.com'))
             errorware['error_message'] = global_conf.get('error_message', 'An internal server error occurred')
         
+        # Standard Pylons configuration directives for Myghty
+        myghty_defaults = {}
+        
+        # Raise a complete error for the error middleware to catch
+        myghty_defaults['raise_error'] = True
+        myghty_defaults['component_root'] = [{os.path.basename(path): path} for \
+                                             path in self.paths['templates']]
+        
         # Merge in the user-supplied Myghty values
         myghty_defaults.update(self.myghty)
+        
+        # Merge additional globals
+        myghty_defaults.setdefault('allow_globals', [])
+                
+        myghty_defaults['allow_globals'].extend(
+            ['c', 'h', 's', 'session', 'request', 'params', 'g', 'render', 'render_fragment']
+        )
+        
         self.myghty = myghty_defaults
         myghty_template_options = {}
         if app_conf.get('cache_dir', False):
-            myghty_defaults['data_dir'] = app_conf['cache_dir']
+            myghty_defaults['data_dir'] = os.path.join(app_conf['cache_dir'], 'templates')
         else:
             myghty_defaults['data_dir'] = app_conf['myghty_data_dir']
         
@@ -195,7 +208,7 @@ class Config(object):
         if not app_conf.has_key('session_data_dir'):
             app_conf['session_data_dir'] = os.path.join(app_conf['cache_dir'], 'sessions')
         if not app_conf.has_key('cache_data_dir'):
-            app_conf['cache_data_dir'] = app_conf['cache_dir']
+            app_conf['cache_data_dir'] = os.path.join(app_conf['cache_dir'], 'cache')
         
         # Prepare our default template engine
         self.add_template_engine('pylonsmyghty', None, myghty_template_options)
