@@ -3,6 +3,7 @@ import types
 import simplejson as json
 import formencode.api as api
 from formencode import htmlfill
+import formencode.variabledecode as variabledecode
 import pylons
 from pylons.decorator import decorator
 import rest
@@ -21,16 +22,17 @@ def jsonify(func, *args, **kw):
     return response
 jsonify = decorator(jsonify)
 
-def validate(schema=None, validators=None, form=None):
+def validate(schema=None, validators=None, form=None, variable_decode=False):
     """Validate input either for a FormEncode schema, or individual validators
     
     Given a form schema or dict of validators, validate will attempt to validate
-    the schema or validator list.
+    the schema or validator list as long as a POST request is made. No 
+    validation is performed on GET requests.
     
-    If validation was succesfull, the valid result dict will be saved as
-     ``self.form_result``. Otherwise, the action will be re-run as if it was a
-     GET, and the output will be filled by FormEncode's htmlfill to fill in the
-     form field errors.
+    If validation was succesfull, the valid result dict will be saved
+    as ``self.form_result``. Otherwise, the action will be re-run as if it was a
+    GET, and the output will be filled by FormEncode's htmlfill to fill in the
+    form field errors.
     
     Example:
     
@@ -51,14 +53,20 @@ def validate(schema=None, validators=None, form=None):
         defaults, errors = {}, {}
         if not pylons.request.method == 'POST':
             return func(self, *args, **kwargs)
-        defaults.update(pylons.request.POST)
+        postvars = pylons.request.POST.copy()
+        if variable_decode:
+            postvars = variabledecode.variable_decode(postvars)
+        
+        defaults.update(postvars)
         if schema:
             try:
                 self.form_result = schema.to_python(defaults)
             except api.Invalid, e:
-                errors = e.unpack_errors()
+                errors = e.unpack_errors(variable_decode)
         if validators:
             if isinstance(validators, dict):
+                if not hasattr(self, 'form_result'):
+                    self.form_result = {}
                 for field, validator in validators.iteritems():
                     try:
                         self.form_result[field] = \
