@@ -9,6 +9,8 @@ information.
 """
 __all__ = []
 
+import cgi
+from pylons.util import get_prefix
 import myghty.exception, sys
 from paste.evalexception.middleware import *
 from paste.exceptions.formatter import *
@@ -168,111 +170,10 @@ class HTMLFormatter(formatter.HTMLFormatter):
             # Usually because another error is already on this page,
             # and so the js & CSS are unneeded
             return text, extra_data
-            
-def format_html(exc_data, include_hidden_frames=False, **ops):
-    if not include_hidden_frames:
-        return HTMLFormatter(**ops).format_collected_data(exc_data)
-    short_er = format_html(exc_data, show_hidden_frames=False, **ops)
-    # @@: This should have a way of seeing if the previous traceback
-    # was actually trimmed at all
-    ops['include_reusable'] = False
-    ops['show_extra_data'] = False
-    long_er = format_html(exc_data, show_hidden_frames=True, **ops)
-    text_er = format_text(exc_data, show_hidden_frames=True, **ops)
-    return """
-    %s
-    <br>
-    <script type="text/javascript">
-    show_button('full_traceback', 'full traceback')
-    </script>
-    <div id="full_traceback" class="hidden-data">
-    %s
-    </div>
-    <br>
-    <script type="text/javascript">
-    show_button('text_version', 'text version')
-    </script>
-    <div id="text_version" class="hidden-data">
-    <textarea style="width: 100%%" rows=10 cols=60>%s</textarea>
-    </div>
-    """ % (short_er, long_er, cgi.escape(text_er)), ''
-
-error_template = '''\
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
-<head>
- <title>Server Error</title>
- %(head)s
-
-<!-- CSS Imports -->
-<link rel="stylesheet" href="%(prefix)s/error/style/orange.css" type="text/css" media="screen" />
-
-<!-- Favorite Icons -->
-<link rel="icon" href="%(prefix)s/error/img/icon-16.png" type="image/png" />
-
-</head>
-
-<body id="documentation" onload="switch_display('%(set_tab)s')">
-<!-- We are only using a table to ensure old browsers see the message correctly -->
-
-<noscript>
-<div style="border-bottom: 1px solid #808080">
-<div style="border-bottom: 1px solid #404040">
-<table width="100%%" border="0" cellpadding="0" bgcolor="#FFFFE1"><tr><td valign="middle"><img src="%(prefix)s/error/img/warning.gif" alt="Warning" /></td><td>&nbsp;</td><td><span style="padding: 0px; margin: 0px; font-family: Tahoma, sans-serif; font-size: 11px">Warning, your browser does not support JavaScript so you will not be able to use the interactive debugging on this page.</span></td></tr></table>
-</div>
-</div>
-</noscript>
-    
-    <!-- Top anchor -->
-    <a name="top"></a>
-    
-    <!-- Logo -->
-    <h1 id="logo"><a class="no-underline" href="http://pylons.groovie.org"><img class="no-border" src="%(prefix)s/error/img/logo.gif" alt="Pylons" title="Pylons"/></a></h1>
-    <p class="invisible"><a href="#content">Skip to content</a></p>
-
-    <!-- Main Content -->
-    <div id="nav-bar">
-
-        <!-- Section Navigation -->
-        <h4 class="invisible">Section Links</h4>
-
-            <ul id="navlist">
-               <!--  %%(links)s -->
-                <li id='traceback_data_tab' class="active"><a href="javascript:switch_display('traceback_data')" id='traceback_data_link' class="active"  accesskey="1">Traceback</a></li>
-                <li id='extra_data_tab' class="" ><a href="javascript:switch_display('extra_data')" id='extra_data_link' accesskey="2" >Extra Data</a></li>
-                <li id='myghty_data_tab'><a href="javascript:switch_display('myghty_data')" accesskey="3" id='myghty_data_link'>Myghty</a></li>
-            </ul>
-    </div>
-    <div id="main-content">
-        <div class="hr"><hr class="hr" /></div>
-        <div class="content-padding">
-            <div id="extra_data" class="hidden-data">
-                %(extra_data)s
-            </div>
-            <div id="myghty_data" class="hidden-data">
-                %(myghty_data)s
-            </div>
-            <div id="traceback_data">
-                %(traceback_data)s
-            </div>
-        </div>
-        <br class="clear" />
-        <div class="hr"><hr class="clear" /></div>
-        <!-- Footer -->
-    </div>
-    <div style=" background: #FFFF99; padding: 10px 10px 10px 6%%">
-        The Pylons Team | 
-        <a href="#top" accesskey="9" title="Return to the top of the navigation links">Top</a>
-    </div>
-</body>
-</html>
-'''
 
 class InvalidTemplate(Exception):
     pass
 
-from pylons.util import get_prefix
-
 class PylonsEvalException(EvalException):
 
     def __init__(self, application, global_conf=None, xmlhttp_key=None,
@@ -304,25 +205,6 @@ class PylonsEvalException(EvalException):
             raise Exception('Invalid template. Please ensure all % signs are properly '
                             'quoted as %% and no extra substitution strings are present.')
             
-    def eval_javascript(self, base_path, counter):
-        base_path += '/_debug' # Note the difference!
-        return (
-            '<script type="text/javascript" src="%s/mochikit/MochiKit.js">'
-            '</script>\n'
-            '<script type="text/javascript" src="%s/media/debug.js">'
-            '</script>\n'
-            '<script type="text/javascript">\n'
-            'debug_base = %r;\n'
-            'debug_count = %r;\n'
-            '</script>\n'
-            % (base_path, base_path, base_path, counter))
-            
-    #~ def pylons(self, environ, start_response):
-        #~ app = urlparser.StaticURLParser(
-            #~ os.path.join(os.path.dirname(__file__), 'media'))
-        #~ return app(environ, start_response)
-    #~ pylons.exposed = True
-    
     def respond(self, environ, start_response):
         if environ.get('paste.throw_errors'):
             return self.application(environ, start_response)
@@ -363,8 +245,9 @@ class PylonsEvalException(EvalException):
             environ['wsgi.errors'].write('Debug at: %s\n' % view_uri)
 
             exc_data = collector.collect_exception(*exc_info)
-            debug_info = DebugInfo(count, exc_info, exc_data, base_path,
-                                   environ, view_uri)
+            #debug_info = PylonsDebugInfo(count, exc_info, exc_data, base_path,
+            debug_info = PylonsDebugInfo(count, exc_info, exc_data, get_prefix(environ),
+                                         environ, view_uri, error_template)
             assert count not in self.debug_infos
             self.debug_infos[count] = debug_info
 
@@ -377,24 +260,208 @@ class PylonsEvalException(EvalException):
                         include_reusable=False, show_extra_data=False)
                     return [html]
             
-            base_path = get_prefix(environ)
-            #base_path = environ['SCRIPT_NAME']
-            
             # @@: it would be nice to deal with bad content types here
-            html, extra_data = format_eval_html(exc_data, base_path, count)
-            #raise Exception(extra_data)
-            head_html = (formatter.error_css + formatter.hide_display_js)
-            head_html += self.eval_javascript(base_path, count)
-            repost_button = make_repost_button(environ)
-            myghty_data = '<p>No Myghty information available.</p>'
-            tab = 'traceback_data'
-            if hasattr(exc_info[1], 'htmlformat'):
-                myghty_data = exc_info[1].htmlformat()[333:-14]
-                tab = 'myghty_data'
-            if hasattr(exc_info[1], 'mtrace'):
-                myghty_data = exc_info[1].mtrace.htmlformat()[333:-14]
-                tab = 'myghty_data'
-            head_html = ("""
+            return debug_info.content()
+
+class PylonsDebugInfo(DebugInfo):
+    def __init__(self, counter, exc_info, exc_data, base_path,
+                 environ, view_uri, error_template):
+        DebugInfo.__init__(self, counter, exc_info, exc_data, base_path,
+                           environ, view_uri)
+        self.error_template = error_template
+
+    def content(self):
+        html, extra_data = format_eval_html(self.exc_data, self.base_path, self.counter)
+        head_html = (formatter.error_css + formatter.hide_display_js)
+        head_html += self.eval_javascript(self.counter)
+        repost_button = make_repost_button(self.environ)
+        myghty_data = '<p>No Myghty information available.</p>'
+        tab = 'traceback_data'
+        if hasattr(self.exc_value, 'htmlformat'):
+            myghty_data = self.exc_value.htmlformat()[333:-14]
+            tab = 'myghty_data'
+        if hasattr(self.exc_value, 'mtrace'):
+            myghty_data = self.exc_value.mtrace.htmlformat()[333:-14]
+            tab = 'myghty_data'
+
+        head_html = (error_head_template % {'prefix':self.base_path}) + head_html
+
+        traceback_data = error_traceback_template % {
+            'prefix':self.base_path,
+            'body':html,
+            'repost_button': repost_button or '',
+        }
+
+        extra_data = """<h1 class="first"><a name="content"></a>Extra Data</h1>""" + \
+            '\n'.join(extra_data)
+        page = self.error_template % {
+            'head': head_html,
+            'traceback_data': traceback_data,
+            'extra_data':extra_data,
+            'myghty_data':myghty_data.replace('<h2>',
+                                              '<h1 class="first">').replace('</h2>',
+                                                                            '</h1>'),
+            'set_tab':tab,
+            'prefix':self.base_path,
+            }
+        return [page]
+
+    def eval_javascript(self, counter):
+        base_path = self.base_path + '/_debug'
+        return (
+            '<script type="text/javascript" src="%s/mochikit/MochiKit.js">'
+            '</script>\n'
+            '<script type="text/javascript" src="%s/media/debug.js">'
+            '</script>\n'
+            '<script type="text/javascript">\n'
+            'debug_base = %r;\n'
+            'debug_count = %r;\n'
+            '</script>\n'
+            % (base_path, base_path, base_path, counter))
+
+class EvalHTMLFormatter(HTMLFormatter):
+
+    def __init__(self, base_path, counter, **kw):
+        super(EvalHTMLFormatter, self).__init__(**kw)
+        self.base_path = base_path
+        self.counter = counter
+    
+    def format_source_line(self, filename, frame):
+        line = formatter.HTMLFormatter.format_source_line(
+            self, filename, frame)
+        return (line +
+                '  <a href="#" class="switch_source" '
+                'tbid="%s" onClick="return showFrame(this)">&nbsp; &nbsp; '
+                '<img src="%s/error/img/plus.jpg" border=0 width=9 '
+                'height=9> &nbsp; &nbsp;</a>'
+                % (frame.tbid, self.base_path))
+
+def format_eval_html(exc_data, base_path, counter):
+    short_formatter = EvalHTMLFormatter(
+        base_path=base_path,
+        counter=counter,
+        include_reusable=False)
+    short_er, extra_data = short_formatter.format_collected_data(exc_data)
+    long_formatter = EvalHTMLFormatter(
+        base_path=base_path,
+        counter=counter,
+        show_hidden_frames=True,
+        show_extra_data=False,
+        include_reusable=False)
+    long_er, extra_data_none = long_formatter.format_collected_data(exc_data)
+    text_er = formatter.format_text(exc_data, show_hidden_frames=True)
+
+    if short_formatter.filter_frames(exc_data.frames) != \
+        long_formatter.filter_frames(exc_data.frames):
+        # Only display the full traceback when it differs from the
+        # short version
+        full_traceback_html = """
+        <br />
+        <div id="util-link">
+            <script type="text/javascript">
+            show_button('full_traceback', 'full traceback')
+            </script>
+        </div>
+        <div id="full_traceback" class="hidden-data">
+        %s
+        </div>
+        """ % long_er
+    else:
+        full_traceback_html = ''
+
+    return """
+    <style type="text/css">
+            #util-link a, #util-link a:link, #util-link a:visited,
+            #util-link a:active {
+                border-bottom: 2px outset #aaa
+            }
+    </style>
+    %s
+    <br />
+    %s
+    <br />
+    <div id="util-link">
+        <script type="text/javascript">
+        show_button('text_version', 'text version')
+        </script>
+    </div>
+    <div id="text_version" class="hidden-data">
+    <textarea style="width: 100%%" rows=10 cols=60>%s</textarea>
+    </div>
+    """ % (short_er, full_traceback_html, cgi.escape(text_er)), extra_data
+
+error_template = '''\
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
+<head>
+ <title>Server Error</title>
+ %(head)s
+
+<!-- CSS Imports -->
+<link rel="stylesheet" href="%(prefix)s/error/style/orange.css" type="text/css" media="screen" />
+
+<!-- Favorite Icons -->
+<link rel="icon" href="%(prefix)s/error/img/icon-16.png" type="image/png" />
+
+</head>
+
+<body id="documentation" onload="switch_display('%(set_tab)s')">
+<!-- We are only using a table to ensure old browsers see the message correctly -->
+
+<noscript>
+<div style="border-bottom: 1px solid #808080">
+<div style="border-bottom: 1px solid #404040">
+<table width="100%%" border="0" cellpadding="0" bgcolor="#FFFFE1"><tr><td valign="middle"><img src="%(prefix)s/error/img/warning.gif" alt="Warning" /></td><td>&nbsp;</td><td><span style="padding: 0px; margin: 0px; font-family: Tahoma, sans-serif; font-size: 11px">Warning, your browser does not support JavaScript so you will not be able to use the interactive debugging on this page.</span></td></tr></table>
+</div>
+</div>
+</noscript>
+    
+    <!-- Top anchor -->
+    <a name="top"></a>
+    
+    <!-- Logo -->
+    <h1 id="logo"><a class="no-underline" href="http://pylonshq.com"><img class="no-border" src="%(prefix)s/error/img/logo.gif" alt="Pylons" title="Pylons"/></a></h1>
+    <p class="invisible"><a href="#content">Skip to content</a></p>
+
+    <!-- Main Content -->
+    <div id="nav-bar">
+
+        <!-- Section Navigation -->
+        <h4 class="invisible">Section Links</h4>
+
+            <ul id="navlist">
+               <!--  %%(links)s -->
+                <li id='traceback_data_tab' class="active"><a href="javascript:switch_display('traceback_data')" id='traceback_data_link' class="active"  accesskey="1">Traceback</a></li>
+                <li id='extra_data_tab' class="" ><a href="javascript:switch_display('extra_data')" id='extra_data_link' accesskey="2" >Extra Data</a></li>
+                <li id='myghty_data_tab'><a href="javascript:switch_display('myghty_data')" accesskey="3" id='myghty_data_link'>Myghty</a></li>
+            </ul>
+    </div>
+    <div id="main-content">
+        <div class="hr"><hr class="hr" /></div>
+        <div class="content-padding">
+            <div id="extra_data" class="hidden-data">
+                %(extra_data)s
+            </div>
+            <div id="myghty_data" class="hidden-data">
+                %(myghty_data)s
+            </div>
+            <div id="traceback_data">
+                %(traceback_data)s
+            </div>
+        </div>
+        <br class="clear" />
+        <div class="hr"><hr class="clear" /></div>
+        <!-- Footer -->
+    </div>
+    <div style=" background: #FFFF99; padding: 10px 10px 10px 6%%">
+        The Pylons Team | 
+        <a href="#top" accesskey="9" title="Return to the top of the navigation links">Top</a>
+    </div>
+</body>
+</html>
+'''
+
+error_head_template = """
 <!-- 
     This is the Pylons error handler.
 
@@ -437,78 +504,22 @@ function switch_display(id) {
     el.className = "active";
 }   
 </script>
-            """%{'prefix':base_path})+head_html
+"""
 
-            traceback_data = """
-            <div style="float: left; width: 100%%; padding-bottom: 20px;">
-            <h1 class="first"><a name="content"></a>Error Traceback</h1>
-            <div id="error-area" style="display: none; background-color: #600; color: #fff; border: 2px solid black">
-            <button onclick="return clearError()">clear this</button>
-            <div id="error-container"></div>
-            <button onclick="return clearError()">clear this</button>
-            </div>
-            %(body)s
-            <br />
-            <div class="highlight" style="padding: 20px;">
-            <b>Extra Features</b>
-            <table border="0">
-            <tr><td>&gt;&gt;</td><td>Display the lines of code near each part of the traceback</td></tr>
-            <tr><td><img src="%(prefix)s/error/img/plus.jpg" /></td><td>Show a debug prompt to allow you to directly debug the code at the traceback</td></tr>
-            </table>
-            </div>%(repost_button)s"""%{
-                'prefix':base_path,
-                'body':html,
-                'repost_button': repost_button or '',
-            }
-        
-            page = self.error_template % {
-                'head': head_html,
-                'traceback_data': traceback_data,
-                'extra_data':"""<h1 class="first"><a name="content"></a>Extra Data</h1>"""+'\n'.join(extra_data),
-                'myghty_data':myghty_data.replace('<h2>','<h1 class="first">').replace('</h2>','</h1>'),
-                'set_tab':tab,
-                'prefix':base_path,
-                }
-            return [page]
-
-class EvalHTMLFormatter(HTMLFormatter):
-
-    def __init__(self, base_path, counter, **kw):
-        super(EvalHTMLFormatter, self).__init__(**kw)
-        self.base_path = base_path
-        self.counter = counter
-    
-    def format_source_line(self, filename, frame):
-        line = formatter.HTMLFormatter.format_source_line(
-            self, filename, frame)
-        return (line +
-                '  <a href="#" class="switch_source" '
-                'tbid="%s" onClick="return showFrame(this)">&nbsp; &nbsp; '
-                '<img src="%s/error/img/plus.jpg" border=0 width=9 '
-                'height=9> &nbsp; &nbsp;</a>'
-                % (frame.tbid, self.base_path))
-
-def format_eval_html(exc_data, base_path, counter):
-    short_er, extra_data = EvalHTMLFormatter(
-        base_path=base_path,
-        counter=counter,
-        include_reusable=False).format_collected_data(exc_data)
-    #raise Exception(extra_data)
-    long_er, extra_data_none = EvalHTMLFormatter(
-        base_path=base_path,
-        counter=counter,
-        show_hidden_frames=True,
-        show_extra_data=False,
-        include_reusable=False).format_collected_data(exc_data)
-    return """
-    %s
-    <br />
-    <br />
-    <script type="text/javascript">
-    show_button('full_traceback', 'full traceback')
-    </script>
-    <div id="full_traceback" class="hidden-data">
-    %s
-    </div>
-    """ % (short_er, long_er), extra_data
-    
+error_traceback_template = """
+        <div style="float: left; width: 100%%; padding-bottom: 20px;">
+        <h1 class="first"><a name="content"></a>Error Traceback</h1>
+        <div id="error-area" style="display: none; background-color: #600; color: #fff; border: 2px solid black">
+        <button onclick="return clearError()">clear this</button>
+        <div id="error-container"></div>
+        <button onclick="return clearError()">clear this</button>
+        </div>
+        %(body)s
+        <br />
+        <div class="highlight" style="padding: 20px;">
+        <b>Extra Features</b>
+        <table border="0">
+        <tr><td>&gt;&gt;</td><td>Display the lines of code near each part of the traceback</td></tr>
+        <tr><td><img src="%(prefix)s/error/img/plus.jpg" /></td><td>Show a debug prompt to allow you to directly debug the code at the traceback</td></tr>
+        </table>
+        </div>%(repost_button)s"""
