@@ -63,13 +63,19 @@ from paste.deploy.converters import asbool
 from pylons.util import get_prefix
 
 def error_mapper(code, message, environ, global_conf=None, **kw):
+    if environ.get('pylons.error_call'):
+        return None
+    else:
+        environ['pylons.error_call'] = True
+    
     if global_conf is None:
         global_conf = {}
     codes = [401, 403, 404]
     if not asbool(global_conf.get('debug')):
         codes.append(500)
     if code in codes:
-        url = '%s/error/document/?%s'%(get_prefix(environ), urlencode({'message':message, 'code':code}))
+        # StatusBasedForward expects a relative URL (no SCRIPT_NAME)
+        url = '/error/document/?%s' % (urlencode({'message':message, 'code':code}))
         return url
 
 def ErrorDocuments(app, global_conf=None, mapper=None, **kw):
@@ -83,8 +89,6 @@ def ErrorDocuments(app, global_conf=None, mapper=None, **kw):
     if mapper is None:
         mapper = error_mapper
     return RecursiveMiddleware(StatusBasedForward(app, global_conf=global_conf, mapper=mapper, **kw))
-    #app = RecursiveMiddleware(app)
-    #return custom_forward(app, global_conf=global_conf, mapper=mapper, **kw)
 
 error_document_template = """\
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
