@@ -22,9 +22,8 @@ try:
     from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
-
-from myghty.interp import Interpreter
-
+import myghty.interp
+import paste.wsgiwrappers
 import pylons
 
 class BuffetError(Exception):
@@ -71,17 +70,18 @@ class Buffet(object):
             dict(engine=Engine(options=config), root=template_root)
     
     def _update_names(self, ns):
-        d = {}
-        d.update(ns)
+        d = ns
         d.update(dict(
-            c=pylons.c,
+            c=pylons.c._current_obj(),
             h=pylons.h,
-            m=pylons.m,
-            request=pylons.request,
-            g=pylons.g,
-            session=pylons.session,
+            request=pylons.request._current_obj(),
+            g=pylons.g._current_obj(),
+            session=pylons.session._current_obj(),
+            s=pylons.session._current_obj(),
             render=render,
         ))
+        if hasattr(d['request'], '_h'):
+            d['h'] = d['request']._h
         return d
     
     def render(self, engine_name=None, template_name=None,
@@ -158,7 +158,23 @@ class Buffet(object):
             if not engine_name.startswith('pylons'):
                 full_path = os.path.join(engine_config['root'], template_name)
                 full_path = full_path.replace(os.path.sep, '.').lstrip('.')
-                
+        else:
+            if namespace is None:
+                namespace = {}
+            namespace['_global_args'] = self._update_names({})
+
+            # Reserved myghty keywords
+            for key in ('output_encoding', 'encoding_errors', 'disable_unicode'):
+                if key in namespace:
+                    options[key] = namespace.pop(key)
+            
+            # If they passed in a variable thats listed in the global_args,
+            # update the global args one instead of duplicating it
+            interp = engine_config['engine'].interpreter
+            for key in interp.global_args.keys() + interp.init_params.get('allow_globals', []):
+                if key in namespace:
+                    namespace['_global_args'][key] = namespace.pop(key)
+        
         # If one of them is not None then the user did set something
         if cache_key is not None or cache_expire is not None or cache_type is not None:
             if not cache_type:
@@ -198,27 +214,25 @@ class MyghtyTemplatePlugin(object):
         for k, v in options.iteritems():
             if k.startswith('myghty.'):
                 myt_opts[k[7:]] = v
-        myt_opts['global_args'] = dict(
-            c=pylons.c,
-            h=pylons.h,
-            request=pylons.request,
-            g=pylons.g,
-            session=pylons.session,
-            s=pylons.session,
-            render=render,
-        )
-        self.interpreter = Interpreter(**myt_opts)
+        self.interpreter = myghty.interp.Interpreter(**myt_opts)
     
     def load_template(self, template_path):
         pass
 
-    def render(self, info, format="html", fragment=False, template=None):
-        vars = info
+    def render(self, info, format="html", fragment=False, template=None,
+               output_encoding=None, encoding_errors=None, disable_unicode=False):
         buf = StringIO()
+        global_args = info.pop('_global_args')
+        optional_args = dict(disable_unicode=disable_unicode)
         if fragment:
-            self.interpreter.execute(template, request_args=vars, out_buffer=buf, disable_wrapping=True)
-        else:
-            self.interpreter.execute(template, request_args=vars, out_buffer=buf)
+            optional_args['disable_wrapping'] = True
+        if output_encoding:
+            optional_args['output_encoding'] = output_encoding
+        if encoding_errors:
+            optional_args['encoding_errors'] = encoding_errors
+        self.interpreter.execute(template, request_args=info,
+                                 global_args=global_args, out_buffer=buf,
+                                 **optional_args)
         return buf.getvalue()
 
 available_engines = {}
@@ -249,12 +263,14 @@ def render(*args, **kargs):
     args = list(args)
     template = args.pop()
     cache_args = dict(cache_expire=kargs.pop('cache_expire', None), 
-                      cache_type=kargs.pop('cache_type', None),
-                      cache_key=kargs.pop('cache_key', None))
+                       cache_type=kargs.pop('cache_type', None),
+                       cache_key=kargs.pop('cache_key', None))
     if args: 
         engine = args.pop()
-        return pylons.buffet.render(engine, template, fragment=fragment, namespace=kargs, **cache_args)
-    return pylons.buffet.render(template_name=template, fragment=fragment, namespace=kargs, **cache_args)
+        return pylons.buffet.render(engine, template, fragment=fragment,
+                                    namespace=kargs, **cache_args)
+    return pylons.buffet.render(template_name=template, fragment=fragment,
+                                namespace=kargs, **cache_args)
 
 def render_response(*args, **kargs):
     """Returns the rendered response within a Response object
@@ -269,7 +285,15 @@ def render_response(*args, **kargs):
             return render_response('/my/template.myt')
     
     """
-    return pylons.Response(render(*args, **kargs))
+    response = pylons.Response(render(*args, **kargs))
+    output_encoding = kargs.get('output_encoding')
+    encoding_errors = kargs.get('encoding_errors')
+    if output_encoding:
+        response.headers['Content-Type'] = '%s; charset=%s' % \
+            (paste.wsgiwrappers.settings['content_type'], output_encoding)
+    if encoding_errors:
+        response.encoding_errors = encoding_errors
+    return response
 
 
 __pudge_all__ = ['render', 'render_response', 'Buffet', 'MyghtyTemplatePlugin']
