@@ -9,6 +9,7 @@ import sys
 import re
 import inspect
 import urllib
+import warnings
 
 import paste.wsgiwrappers
 import paste.httpexceptions as httpexceptions
@@ -20,7 +21,7 @@ from routes import request_config
 import pylons
 import pylons.templating
 import pylons.legacy
-from pylons.util import RequestLocal, class_name_from_module_name
+from pylons.util import ContextObj, AttribSafeContextObj, _Translator, class_name_from_module_name
 from pylons.controllers import Controller, WSGIController
 
 class PylonsBaseWSGIApp(object):
@@ -40,14 +41,20 @@ class PylonsBaseWSGIApp(object):
     "monkey-patching" this class. Subclassing is the preferred approach.
     
     """
-    def __init__(self, mapper, package_name, globals, default_charset='UTF-8'):
+    def __init__(self, mapper, package_name, globals, default_charset=None):
         self.mapper = mapper
         self.globals = globals
         self.package_name = package_name
-        self.default_charset = default_charset
-        self.settings = dict(charset=default_charset, content_type='text/html')
-        self.c = RequestLocal()
         config = globals.pylons_config
+        if default_charset:
+            warnings.warn(
+                "The 'default_charset' keyword argument to the PylonsBaseWSGIApp "
+                "constructor is deprecated. Please specify 'default_charset' to the Config "
+                'object in your config/environment.py file instead, e.g.:\n'
+                'return pylons.config.Config(myghty, map, paths, '
+                "default_charset='%s')" % default_charset, DeprecationWarning, 2)
+            config.default_charset = default_charset
+        self.settings = dict(content_type='text/html', charset=config.default_charset)
         
         # Create the redirect function we'll use and save it
         def redirect_to(url):
@@ -70,12 +77,12 @@ class PylonsBaseWSGIApp(object):
         
         # Change our HTTP_METHOD if _method is present, try GET first to avoid
         # parsing POST unless absolutely necessary.
-        req = pylons.request.current_obj()
+        req = pylons.request._current_obj()
         old_method = None
-        if req.GET.has_key('_method'):
+        if '_method' in environ['QUERY_STRING'] and req.GET.has_key('_method'):
             old_method = environ['REQUEST_METHOD']
             environ['REQUEST_METHOD'] = req.GET['_method']
-        elif req.POST.has_key('_method'):
+        elif environ['REQUEST_METHOD'] == 'POST' and req.POST.has_key('_method'):
             old_method = environ['REQUEST_METHOD']
             environ['REQUEST_METHOD'] = req.POST['_method']
         
@@ -110,20 +117,55 @@ class PylonsBaseWSGIApp(object):
     def setup_app_env(self, environ, start_response):
         """Setup and register all the Pylons objects with the registry"""
         req = WSGIRequest(environ)
+        
+        # This is a legacy test for pre-0.9.2 projects to continue using the old
+        # style Helper. New style directly uses the projects lib.helpers module
+        # for 'normal' Python import usage with no surprises.
+        __import__(self.package_name + '.lib.base')
+        their_h = getattr(sys.modules[self.package_name + '.lib.base'], 'h', None)
+
+        try: 
+            helpers_name = self.package_name + '.config.helpers' 
+            __import__(helpers_name) 
+        except: 
+            helpers_name = self.package_name + '.lib.helpers' 
+            __import__(helpers_name)
+        if their_h != pylons.h and their_h is not None:
+            # We're using the new Helper import style
+            req._h = sys.modules[helpers_name]
+        else:
+            # We still need to keep a reference to their helpers for the old
+            # Helpers object.
+            req._oldh = sys.modules[helpers_name]
+        
+        # Setup the translator global object
+        trans = dict(translator=_Translator())
+        trans['CONFIG'] = dict(app_conf=self.globals.pylons_config.app_conf,
+            global_conf=self.globals.pylons_config.global_conf)
+        if self.globals.pylons_config.app_conf.has_key('lang'):
+            trans['translator'](self.globals.pylons_config.app_conf['lang'])
+        else:
+            trans['lang'] = None
+        environ['paste.registry'].register(pylons.translator, trans)
+        
+        # Setup the basic pylons global objects
         environ['paste.registry'].register(paste.wsgiwrappers.settings, self.settings)
         environ['paste.registry'].register(pylons.request, req)
-        environ['paste.registry'].register(pylons.c, self.c)
+        if self.globals.pylons_config.strict_c:
+            environ['paste.registry'].register(pylons.c, ContextObj())
+        else:
+            environ['paste.registry'].register(pylons.c, AttribSafeContextObj())
         environ['paste.registry'].register(pylons.g, self.globals)
         environ['paste.registry'].register(pylons.buffet, self.buffet)
-        pylons.h()
-        self.c._clear()
         
         # Setup legacy globals
         if environ.get('pylons.legacy'):
+            # Legacy mixed dictionary instead of MultiDict
+            req._legacy_params = req.params.mixed()
             environ['paste.registry'].register(pylons.legacy.response, WSGIResponse())
             environ['paste.registry'].register(pylons.m, 
                 pylons.legacy.Myghty_Compat(environ, start_response))
-            environ['paste.registry'].register(pylons.params, req.params)
+            environ['paste.registry'].register(pylons.params, req._legacy_params)
         
         econf = environ['pylons.environ_config']
         if econf.get('session'):
@@ -152,7 +194,7 @@ class PylonsBaseWSGIApp(object):
             + controller.replace('/', '.')
         
         __import__(full_module_name)
-        module_name = full_module_name.split('.')[-1]
+        module_name = controller.split('/')[-1]
         class_name = class_name_from_module_name(module_name) + 'Controller'
         return getattr(sys.modules[full_module_name], class_name)
         
@@ -208,9 +250,9 @@ class PylonsBaseWSGIApp(object):
     def load_test_env(self, environ):
         """Sets up our Paste testing environment"""
         testenv = environ['paste.testing_variables']
-        testenv['req'] = pylons.request.current_obj()
-        testenv['c'] = pylons.c.current_obj()
-        testenv['g'] = pylons.g.current_obj()
+        testenv['req'] = pylons.request._current_obj()
+        testenv['c'] = pylons.c._current_obj()
+        testenv['g'] = pylons.g._current_obj()
         econf = environ['pylons.environ_config']
         if econf.get('session'):
             testenv['session'] = environ[econf['session']]
@@ -228,8 +270,17 @@ class PylonsApp(object):
     then no session/cache objects will be available.
     
     """
-    def __init__(self, config, default_charset='UTF-8'):
+    def __init__(self, config, default_charset=None):
         self.config = config
+        if default_charset:
+            warnings.warn(
+                "The 'default_charset' keyword argument to the PylonsApp constructor is "
+                "deprecated. Please specify 'default_charset' to the Config object in your "
+                'config/environment.py file instead, e.g.:\n'
+                'return pylons.config.Config(myghty, map, paths, '
+                "default_charset='%s')" % default_charset, DeprecationWarning, 2)
+            self.config.default_charset = default_charset
+
         g = None
         try:
             package = __import__(config.package + '.lib.app_globals', globals(), locals(), ['Globals'])
@@ -240,7 +291,7 @@ class PylonsApp(object):
             g.pylons_config = config
         
         # Create the base Pylons wsgi app
-        app = PylonsBaseWSGIApp(config.map, config.package, g, default_charset)
+        app = PylonsBaseWSGIApp(config.map, config.package, g)
         
         # Pull user-specified environ overrides, or just setup default
         # session and caching objects
@@ -278,6 +329,11 @@ class LegacyApp(object):
 
 def make_app(config):
     """ Legacy WSGI app creator"""
+    warnings.warn(
+        "Legacy WSGI app in use for pre-0.9 application. This will be "
+        "removed at some point post-1.0 which will require minor updates "
+        "to your application.",
+        DeprecationWarning, 2)
     papp = LegacyApp(config)
     from paste.deploy.config import ConfigMiddleware
     app = ConfigMiddleware(papp, {
